#include "IZ_app.h"

IZ_ProcedureResult IZ_AppInitialize(IZ_App* app) {
	u32 flags = (
		SDL_INIT_VIDEO
		| SDL_INIT_GAMECONTROLLER
		| SDL_INIT_EVENTS
	);

	if (SDL_Init(flags) < 0) {
		// TODO fix logging
		fprintf_s(stderr, "SDL could not initialize! SDL_Error: %s\n", SDL_GetError());
		return 1;
	}

	char config_path[128];
	IZ_ConfigGetPath(config_path, 128);
	if (IZ_VideoInitialize(config_path, &app->video_state)) {
		return 2;
	}

	IZ_InputInitialize(config_path, &app->input_state);
	IZ_PoolInitialize(&app->pool);
//	void* p1 = IZ_PoolAllocate(&app->memory_pool, sizeof(u16), 0)->pointer;
//	void* p2 = IZ_PoolAllocate(&app->memory_pool, sizeof(u8), 0)->pointer;
//	void* p3 = IZ_PoolAllocate(&app->memory_pool, sizeof(u64), 0)->pointer;
//	void* p4 = IZ_PoolAllocate(&app->memory_pool, sizeof(u32), 0)->pointer;
//	printf("\n%p %p %p %p\n", p1, p2, p3, p4);
//	IZ_PoolDeallocate(p1);
//	void* p5 = IZ_PoolAllocate(&app->memory_pool, sizeof(u16), 0)->pointer;
//	printf("\n%p\n", p5);

  // TODO put into its timer module
	app->ticks = 0;
	app->quit = false;
	return 0;
}

void IZ_AppTeardown(IZ_App* app) {
	IZ_PoolTeardown(&app->pool);
	IZ_InputTeardown(&app->input_state);
	IZ_VideoTeardown(&app->video_state);
	SDL_Quit();
}

void IZ_AppHandleSDLEvents(IZ_App* app) {
	while (SDL_PollEvent(&app->sdl_event) != 0) {
		if (app->sdl_event.type == SDL_QUIT) {
			app->quit = true;
			break;
		}

		IZ_InputHandleSDLEvents(app->sdl_event, &app->input_state);
	}
}

void IZ_AppHandlePortMIDIEvents(IZ_App* app) {
	u8 player_index;
	i32* midi_events_count;
	u32 midi_event_index;
	for (player_index = 0; player_index < PLAYERS; player_index += 1) {
		midi_events_count = &app->input_state.midi_input_state[player_index].midi_events_count;
		*midi_events_count = Pm_Read(
			app->input_state.midi_input_state[player_index].stream,
			app->input_state.midi_input_state[player_index].event_buffer,
			1024
		);

		if (*midi_events_count < 1) {
			continue;
		}

		for (midi_event_index = 0; midi_event_index < *midi_events_count; midi_event_index += 1) {
			IZ_InputHandlePortMIDIEvents(
				app->input_state.midi_input_state[player_index].event_buffer[midi_event_index],
				&app->input_state
			);
		}
	}
}

void IZ_AppHandleEvents(IZ_App* app) {
	IZ_AppHandleSDLEvents(app);
	IZ_AppHandlePortMIDIEvents(app);
}

IZ_ProcedureResult IZ_AppRun(IZ_App* app, u8 arg_count, char* arg_values[]) {
	printf_s("Args (%u):\n", arg_count);
	u8 arg_index;
	for (arg_index = 0; arg_index < arg_count; arg_index += 1) {
		printf_s("  %s\n", arg_values[arg_index]);
	}


	IZ_ProcedureResult init_result = IZ_AppInitialize(app);
	if (init_result) {
		return init_result;
	}

	while (true) {
		app->ticks = SDL_GetTicks64();

		// TODO do audio processing
		// TODO do networking?

		IZ_AppHandleEvents(app);
		if (app->quit) {
			break;
		}

		IZ_VideoUpdate(&app->video_state, &app->input_state, app->ticks);
	}

	IZ_AppTeardown(app);
	return 0;
}
