#include "IZ_pool.h"

void IZ_PoolInitialize(IZ_Pool* pool) {
	SDL_memset(pool->items, 0, POOL_MAX_ITEMS * sizeof(IZ_PoolItem));
	pool->top = 0;
	pool->memory = SDL_malloc(POOL_MAX_SIZE);
	SDL_memset(pool->memory, 0, POOL_MAX_SIZE);
	pool->next_address = 0;
}

IZ_PoolItem* IZ_PoolAllocate(IZ_Pool* pool, size_t size, u64 created_at) {
	// 1. check next free allocation for size
	// 2. if 1. returns non-null,

//	u64 alloc_end = pool->next_address + size;
//
//	for (u64 i = 0; i < POOL_MAX_ALLOCATIONS; i += 1) {
//		if (pool->allocation[i].length == 0) {
//			continue;
//		}
//	}

	void* pointer = &pool->memory[pool->next_address];
	IZ_PoolItem* next_allocation = &pool->items[pool->top];
	pool->items[pool->top] = (IZ_PoolItem) {
		.pointer = pointer,
		.size = size,
		.created_at = created_at,
		.pool = pool,
	};
	pool->top = (pool->top + 1) % POOL_MAX_ITEMS;
	pool->next_address = (pool->next_address + size) % POOL_MAX_SIZE;
	return next_allocation;
}

void IZ_PoolDeallocate(IZ_PoolItem* item) {
	u64 i;
	for (i = 0; i < POOL_MAX_ITEMS; i += 1) {
		if (&item->pool->items[i] != item) {
			continue;
		}
		item->pool->items[i].pool = NULL;
		item->pool->items[i].size = 0;
		item->pool->items[i].pointer = NULL;
		item->pool->items[i].created_at = 0;
		return;
	}
}

void IZ_PoolTeardown(IZ_Pool* pool) {
	SDL_free(pool->memory);
}
